@@ -1,6 +1,6 @@
 package Gtk2::Ex::HyperTextBuffer;
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 
 use strict;
 use Gtk2;
@@ -83,6 +83,10 @@ sub INIT_INSTANCE {
 		} );
 	$self->signal_connect_after(insert_text => \&on_insert);
 	$self->signal_connect(delete_range => \&on_delete_range);
+	$self->signal_connect(apply_tag => \&on_apply_tag);
+	$self->signal_connect(remove_tag => \&on_remove_tag);
+	$self->signal_connect(begin_user_action => sub {$$self{_ua} = 1});
+	$self->signal_connect(end_user_action => sub {$$self{_ua} = 0});
 }
 
 =item C<clear()>
@@ -642,12 +646,15 @@ sub toggle_edit_mode_tag {
 
 sub on_insert {
 	my ($self, $end, $string) = @_;
+
 	# length in argument list does not understand utf8
 	my $start = $end->copy;
 	$start->backward_chars(length $string);
+
+	$self->_record_step('I', $start->get_offset, $end->get_offset, $string);
+		# record this step _before_ triggering any steps from tags
 	$self->remove_all_tags($start, $end);
 	$self->apply_tag($_, $start, $end) for @{$self->{edit_mode_tags}};
-	$self->_record_step('I', $start->get_offset, $end->get_offset, $string);
 }
 
 sub on_delete_range {
@@ -656,62 +663,104 @@ sub on_delete_range {
 	$self->_record_step('D', $begin->get_offset, $end->get_offset, $tree);
 }
 
+sub on_apply_tag {
+	my ($self, $tag, $begin, $end) = @_;
+	$self->_record_step('A', $begin->get_offset, $end->get_offset, $tag);
+}
+
+sub on_remove_tag {
+	my ($self, $tag, $begin, $end) = @_;
+	$self->_record_step('R', $begin->get_offset, $end->get_offset, $tag);
+}
+
 =item C<undo>
 
-TODO
+Undo one editing step.
 
 =item C<redo>
 
-TODO
+Redo one editing step.
 
 =cut
 
-my %reverse_step = (
-	I => 'D',
-	D => 'I',
-	A => 'R',
-	R => 'A',
-);
+my %_reverse_step = qw/I D D I A R R A/; # two pairs of complementary actions
 
 sub undo {
 	my $self = shift;
-	return unless @{$self->{undo}};
+	return 0 unless @{$self->{undo}};
 	my $step = pop @{$self->{undo}};
 	push @{$self->{redo}}, [@$step];
-	$$step[0] = $reverse_step{$$step[0]};
-	$self->_do_step(@$step);
+	$self->_do_step($step, 1);
+	return 1;
 }
 
 sub redo {
 	my $self = shift;
-	return unless @{$self->{redo}};
+	return 0 unless @{$self->{redo}};
 	my $step = pop @{$self->{redo}};
 	push @{$self->{undo}}, [@$step];
-	$self->_do_step(@$step);
+	$self->_do_step($step, 0);
+	return 1;
 }
 
 sub _record_step {
 	my ($self, $step, $begin, $end, @data) = @_;
-	return if $self->{_undo_lock};
-	$self->{_redo} = [];
+	#return unless $$self{_ua} and !$$self{_undo_lock};
+	return if $$self{_undo_lock};
+	#warn "# Record ($$self{_ua}): $step, $begin, $end, @data\n";
+	$self->_fold_steps if @{$self->{redo}};
 	push @{$self->{undo}}, [$step, $begin, $end, @data];
 }
 
+sub _fold_steps {
+	# Triggered if you start editing after an undo
+	# "folds" the undo sequence instead of throwing it away
+	my $self = shift;
+	my @steps = reverse @{$self->{redo}}; # reverse because we pop in undo()
+
+	# redo the whole stack
+	$self->{redo} = [];
+	push @{$self->{undo}}, @steps;
+
+	# add the "undo" action as undoable action
+	@steps = reverse map [@$_], @steps; # copy, no ref, reverse again
+	$$_[0] = $_reverse_step{$$_[0]} for @steps;
+	push @{$self->{undo}}, [@steps]; # push whole group as single step
+}
+
 sub _do_step {
-	my ($self, $step, $begin, $end, @data) = @_;
+	# does one step or one group of steps
+	my ($self, $ref, $reverse) = @_;
+	my @steps = (ref($$ref[0]) ? @$ref : $ref);
+	@steps = reverse @steps if $reverse;
+	
 	$self->{_undo_lock} = 1;
-	$_ = $self->get_iter_at_offset($_) for $begin, $end;
-	if ($step eq 'I') { # insert
-		$self->insert_blocks($begin, @data);
+	for my $step (@steps) {
+		my ($act, $begin, $end, @data) = @$step;
+		$act = $_reverse_step{$act} if $reverse;
+		#warn "Do: $act, $begin, $end, @data\n";
+		$_ = $self->get_iter_at_offset($_) for $begin, $end;
+		if ($act eq 'I') { # insert
+			$self->place_cursor($begin);
+			$self->insert_blocks_at_cursor(@data);
+		}
+		elsif ($act eq 'D') { # delete
+			my $tree = $self->get_parse_tree($begin, $end);
+			splice @$tree, 3, 0, $tree;
+				# replace insert data with actual deleted data
+				# these should be the same
+			$self->delete($begin, $end);
+			$self->place_cursor($end);
+		}
+		elsif ($act eq 'A') { # apply tag
+			$self->apply_tag($data[0], $begin, $end);
+		}
+		elsif ($act eq 'R') { # remove tag
+			$self->remove_tag($data[0], $begin, $end);
+		}
+		else { die "BUG: unknown step type: $act" }
 	}
-	elsif ($step eq 'D') { # delete
-		$self->delete($begin, $end);
-	}
-	elsif ($step eq 'A') { # add tag
-	}
-	elsif ($step eq 'R') { # remove tag
-	}
-	else { die "BUG: unknown step type: $step" }
+	$self->set_modified(1);
 	$self->{_undo_lock} = 0;
 }
 
