@@ -4,16 +4,21 @@ use strict;
 use Carp;
 use UNIVERSAL qw/isa/;
 use File::Spec;
-use File::BaseDir qw/xdg_config_files xdg_data_home xdg_data_dirs xdg_data_files/;
+use File::BaseDir qw/
+	xdg_config_home xdg_config_files
+	xdg_data_home xdg_data_dirs xdg_data_files/;
+use Zim::Store;
 use Zim::Selection;
+use Zim::History;
 use Zim::File;
 use Zim::Page;
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
+our $COPYRIGHT = 'Copyright (c) 2005,2007 Jaap G Karssenberg.';
 our $LONG_VERSION = << "EOT";
 zim $VERSION - A desktop wiki and outliner
 
-Copyright (c) 2006 Jaap G Karssenberg. All rights reserved.
+$COPYRIGHT All rights reserved.
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl.
 
@@ -23,6 +28,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 Please report bugs to pardus\@cpan.org
 EOT
+our $AUTHORS = 'Jaap Karssenberg <pardus@cpan.org>';
+our $WEBSITE = 'http://www.pardus.nl/projects/zim';
 
 =head1 NAME
 
@@ -32,18 +39,21 @@ Zim - Application object for the zim desktop wiki
 
 	use Zim;
 	
-	my $repo = Zim->new(dir => $dir);
+	my $repo = Zim->new(
+		type     => 'Files',
+		dir      => $dir,
+	);
 	my $page = $repo->get_page('Home');
 
 =head1 DESCRIPTION
 
 This class defines the public interface to the document repository
 as used by L<zim>(1). It acts as a dispatcher that manages one or
-more child objects to handle page requests. It also adds some tests
-and thus tries to isolate the handlers from errors in the GUI.
+more store objects to handle page requests. It also adds some tests
+and thus tries to isolate the stores from errors in the GUI.
 
 The interface which child objects should support is defined in
-L<Zim::Repository>.
+L<Zim::Store>.
 
 =head1 METHODS
 
@@ -86,32 +96,102 @@ sub new {
 	my $self = bless {%param}, $class;
 	$self->{config} ||= {};
 	
+	# Find config
 	my ($conf) = grep $_->exists(),
 	             map  Zim::File->new($param{dir}, $_),
 		     qw/_zim.config .zim.config/;
 	$conf ||= Zim::File->new($param{dir}, '.zim.config');
-	$self->{conf_file} = $conf->path;
-	for ($conf->read) {
-		s/[\n\r]+$//;
-		next unless /\S/;
-		my ($opt, $val) = split '=', $_, 2;
-		if ($opt =~ /^:/) {
-			# opt is a namespace with sub-repository
-			# allow e.g. ":namespace=Class,key=val,key=val"
-			my ($class, $arg) = split ',', $val, 2;
-			my %arg = map split('=',$_,2),
-			          map split(',',$_,2), $arg; 
-			$self->add_child($opt, $class, %arg);
-		}
-		else { $self->{config}{$opt} = $val }
+	$self->{conf_file} = $conf;
+
+	# Read config
+	$conf->read_config( $self->{config} );
+	for my $ns (grep /^:/, keys %{$self->{config}}) {
+		# remove namespace defnitions from config
+		# allows lines like ":namespace=Class,key=val,key=val"
+		my $val = delete $self->{config}{$ns};
+		my ($class, $arg) = split ',', $val, 2;
+		my %arg = map split('=',$_,2),
+		          map split(',',$_,2), $arg; 
+		$self->add_child($ns => $class, %arg);
 	}
 	
-	# default config
+	# Set default store if not yet defined
 	my $type = $self->{type} || 'Files';
 	$self->add_child(':', $type, %param)
 		unless defined $self->{':'};
+	$self->{':'}{indexpage} = $self->{config}{home} || '';
 
-	return $self;
+	return $self;	
+}
+
+=item C<init_history()>
+
+Initializes a L<Zim::History> to be used with this repository.
+Returns the history object or undef.
+
+=cut
+
+sub init_history {
+	my $self = shift;
+
+	my $hfile = $self->{config}{hist_file};
+	unless ($hfile) { # default hist file
+		$hfile = Zim::File->abs_path(
+			$self->{dir}.'/.zim.history.cache');
+		$hfile = Zim::File->cache_path($hfile)
+			if -e $self->{dir} and ! -w $self->{dir};
+	}
+	
+	my $hdir = Zim::File->dir($hfile);
+	$hfile = undef
+		if (-e $hfile and ! -w $hfile)
+		or (-e $hdir  and ! -w $hdir ) ;
+	warn "# History file: $hfile\n";
+	my $max = $self->{config}{hist_max} || 15;
+	eval {
+		$self->{history} ||= Zim::History->new($hfile, $max, undef);
+	};
+	warn $@ if $@;
+	
+	return $self->{history};
+}
+
+=item C<add_child(NAMESPACE, CHILD, ...)>
+
+This will connect a new child object to this repository under a certain
+namespace. CHILD is a class name implementing the repository interface.
+The class will be looked for in C<Zim::Store::>.
+All remaining arguments will be passed on to the constructor of the object.
+
+=cut
+
+sub add_child {
+	my ($self, $namespace, $child, %args) = @_;
+	$namespace =~ s/:?$/:/;
+	_check_namespace($namespace);
+	warn "# Adding store of type '$child' at '$namespace'\n";
+	$args{namespace} = $namespace;
+
+	my $class = "Zim::Store::" .
+		join('::', map quotemeta($_), split '::', $child);
+	eval "use $class ();";
+	die "While loading store '$child':\n".$@ if $@;
+	my $obj = $class->new(parent => $self, %args);
+
+	$namespace =~ s/:*$//;
+		# strip last ":" because we also want the page
+		# of the same name as the namespace to resolve to this
+		# child, this page is known as the "indexpage"
+	$namespace ||= ':';
+	$self->{lc($namespace)} = $obj;
+
+	my $ns = $self->{namespace};
+	$ns =~ s/:*$//;
+	my $regex = join '|', map quotemeta($_),
+		sort {length($b) <=> length($a)} grep /^:./, keys %$self;
+	$regex = length($regex) ? "(?:$regex)(?![^:])|:" : ':';
+	$self->{regex} = qr#^(?i)\Q$ns\E($regex)#;
+	#warn "regex: $self->{regex}\n";
 }
 
 =item C<config()>
@@ -122,22 +202,31 @@ Returns a hash with config options.
 
 sub config { $_[0]->{config} }
 
-=item C<save_config()>
+=item C<save()>
 
-Saves the config to disk.
+Save config, history, etc. to disk.
 
 =cut
 
-sub save_config {
+sub save {
+	# Need to merge config with definition for stores
 	my $self = shift;
-	my $node = $self->{dir};
-	my $conf = Zim::File->new($self->{conf_file});
-	warn "Saving $self->{conf_file}\n";
-	my $text = join '', map "$_=$$self{config}{$_}\n", keys %{$$self{config}};
-	for ($conf->read) { $text .= $_ if /^:/ }
-	$conf->write($text);
+	my $file = Zim::File->new($self->{conf_file});
+	my $conf = $file->read_config;
+	! /^:/ && delete $$conf{$_} for keys %$conf;
+	$$conf{$_} = $$self{config}{$_} for keys %{$$self{config}};
+	$file->write_config($conf);
+	$self->{history}->write if $self->{history};
 }
 
+=item C<root()>
+
+Returns self.
+
+=cut
+
+sub root { return $_[0] }
+
 =item C<base()>
 
 Returns the base directory for files.
@@ -190,16 +279,24 @@ sub list_pages {
 	my ($self, $namespace) = @_;
 	$namespace =~ s/:?$/:/;
 	_check_namespace($namespace);
+	
+	# List stores in this namespace
 	my @pages =
 		grep { m/^:/ and s/^\Q$namespace\E:*([^:]+:*)$/$1:/ } keys %$self;
+	@pages = map { s/.*://; $_ } map $self->{$_}{namespace}, @pages;
+
+	# List pages from the store managing this namespace
 	$namespace =~ $self->{regex}
 		or die "BUG: '$namespace' !~ qr/$self->{regex}/";
 	return if $self->{lc($1)}->{no_show_in_sidepane}; # temp HACK
 	@pages = sort @pages, $self->{lc($1)}->list_pages($namespace);
-	for (0 .. $#pages-1) { # remove doubles etc.
+
+	# remove doubles
+	for (0 .. $#pages-1) {
 		$pages[$_] = undef if $pages[$_+1] eq $pages[$_];
 		$pages[$_] = undef if $pages[$_+1] eq $pages[$_].':';
 	}
+
 	return sort {lc($a) cmp lc($b)} grep defined($_), @pages;
 }
 
@@ -233,7 +330,7 @@ sub resolve_page {
 	return $name ? $self->get_page($name) : undef ;
 }
 
-=item C<resolve_name(LINK, PAGE, NO_DEFAULT)>
+=item C<resolve_name(LINK, PAGE, REFERENCE, NO_DEFAULT)>
 
 Cleans up NAME but does not actually fetch a page object.
 
@@ -241,20 +338,23 @@ Cleans up NAME but does not actually fetch a page object.
 
 sub resolve_name {
 	my ($self, $name, $ref, $no_def) = @_;
+	#warn "!! resolve name: @_\n";
 	if ($ref) {
 		# TODO check for multiple children in path
 		# TODO use child path case for path
-		$name = Zim::Repository->clean_name($name, 1) or return;
+		$name = Zim::Store->clean_name($name, 1) or return;
 		$ref =~ $self->{regex}
 			or die "BUG: '$ref' !~ qr/$self->{regex}/";
+		#warn "!! dispatch to: $1\n";
 		return $self->{lc($1)}->resolve_name($name, $ref, $no_def);
 	}
 	else {
-		$name = Zim::Repository->clean_name($name, 0) or return;
+		$name = Zim::Store->clean_name($name, 0) or return;
 		$name =~ $self->{regex}
 			or die "BUG: '$name' !~ qr/$self->{regex}/";
 		my $child = $1;
 		$name =~ s/^$child/$child/i; # set correct caps
+		#warn "!! dispatch to: $1\n";
 		return $self->{lc($child)}->resolve_name($name, undef, $no_def);
 	}
 }
@@ -267,7 +367,7 @@ Returns a namespace string. Used to sanitize user input.
 
 sub resolve_namespace {
 	my ($self, $name) = @_;
-	$name = Zim::Repository->clean_name($name);
+	$name = Zim::Store->clean_name($name);
 	$name .= ':'; 	# is a namespace
 	$name =~ $self->{regex} or die "BUG: '$name' !~ qr/$self->{regex}/";
 	my $child = $1;
@@ -311,10 +411,10 @@ sub _clone { # Copy and move are almost the same
 		if $target->exists and ! $target->equals($source);
 		# if pages are equal the case can still differ
 	
-	if ($source->{repository} eq $target->{repository}
-		and $target->{repository} ne $self
+	if ($source->{store} eq $target->{store}
+		and $target->{store} ne $self
 	) {
-		$source->{repository}->$method($source, $target, $update);
+		$source->{store}->$method($source, $target, $update);
 	}
 	else {
 		$target->clone($source);
@@ -323,7 +423,7 @@ sub _clone { # Copy and move are almost the same
 			$self->get_page($_)->update_links($from => $to)
 				for $source->list_backlinks ;
 		}
-		$source->{repository}->delete_page($source)
+		$source->{store}->delete_page($source)
 			if $method eq 'move_page';
 	}
 
@@ -342,29 +442,15 @@ sub delete_page {
 	$page = $self->get_page($page) unless ref $page;
 	die "BUG: You tried to delete a page marked as read-only\n"
 		if $page->{properties}{read_only};
-	if ($page->{repository} eq $self) {
+	if ($page->{store} eq $self) {
 		$page->status('deleted');
 	}
 	else {
-		$page->{repository}->delete_page($page);
+		$page->{store}->delete_page($page);
 	}
 	return $page;
 }
 
-=item C<resolve_link(PAGE, NAME)>
-
-Returns a page object for NAME where NAME can be resolved
-relative to PAGE.
-
-=cut
-
-sub resolve_link {
-	my ($self, $page, $name) = @_;
-	_check_page($page, 1);
-	return $self->resolve_page($name) if $page->{repository} eq $self;
-	return $page->{repository}->resolve_page($page, $name);
-}
-
 =item C<list_backlinks(PAGE)>
 
 Returns a list of links to this page.
@@ -382,14 +468,6 @@ sub list_backlinks {
 	return @backlinks;
 }
 
-=item C<root()>
-
-Returns paret or self.
-
-=cut
-
-sub root { return $_[0] }
-
 =item C<search(QUERY, CALLBACK)>
 
 TODO stable api for this using Zim::Selection
@@ -409,51 +487,6 @@ sub search {
 	}
 }
 
-=back
-
-=head2 Utility Methods
-
-These methods can be used by the interface but do not need to be reproduced
-in child handlers.
-
-=over 4
-
-=item C<add_child(NAMESPACE, CHILD, ...)>
-
-This will connect a new child object to this repository under a certain
-namespace. CHILD is a class name implementing the repository interface.
-The class will be looked for in C<Zim::Repository::>.
-All remaining arguments will be passed on to the constructor of the object.
-
-=cut
-
-sub add_child {
-	my ($self, $namespace, $child, %args) = @_;
-	$namespace =~ s/:?$/:/;
-	_check_namespace($namespace);
-	warn "# Adding child repository of type '$child' at '$namespace'\n";
-	
-	my $class = "Zim::Repository::" .
-		join('::', map quotemeta($_), split '::', $child);
-	#warn "use $child ();\n";
-	eval "use $class ();";
-	die "While loading child repository '$child' for namespace '$namespace' :\n".$@ if $@;
-	$args{namespace} = $namespace;
-	my $obj = $class->new(parent => $self, %args);
-
-	$namespace =~ s/:*$//;
-	$namespace = ':' if $namespace eq '';
-	$self->{lc($namespace)} = $obj;
-
-	my $ns = $self->{namespace};
-	$ns =~ s/:*$//;
-	my $regex = join '|', map quotemeta($_),
-		sort {-(length($a) <=> length($b))} grep /^:./, keys %$self;
-	$regex = length($regex) ? "(?:$regex)(?![^:])|:" : ':';
-	$self->{regex} = qr#^(?i)\Q$ns\E($regex)#;
-	#warn "regex: $self->{regex}\n";
-}
-
 =item C<interwiki_lookup(KEY, PAGE)>
 
 Returns an url for an interwiki link with interwiki name KEY and page PAGE.
@@ -472,9 +505,9 @@ sub interwiki_lookup {
 		# TODO repository specific urls here
 		
 		# Lookup named zim repositories
-		my $file = $class->lookup_by_name($key);
-		if (defined $file) {
-			$url = Zim::File->path2uri($file);
+		my $dir = $class->get_repository($key);
+		if (defined $dir) {
+			$url = Zim::File->path2uri($dir);
 			$url =~ s#^file:#zim:#i;
 			$url .= '?{NAME}';
 		}
@@ -498,19 +531,16 @@ sub interwiki_lookup {
 	return $url;
 }
 
-=item C<split_page(PAGE, LEVEL)>
+=item C<get_history()>
 
-Splits a page into multiple pages in the like-named namespace.
-Splits the page at headings up to level LEVEL (1..5 default 1).
-
-TODO
-
-=item C<merge_page(NAME)>
-
-TODO - work in progress
+Return a object of class L<Zim::History> or undef.
 
 =cut
 
+sub get_history { $_[0]{history} }
+
+####
+
 sub _flush_cache { # TODO TODO good API for this
 	my $self = shift;
 	for (grep /^:/, keys %$self) {
@@ -527,25 +557,74 @@ These methods can be called as C<< Zim->method(...) >>.
 
 =over 4
 
-=item C<lookup_by_name(NAME)>
+=item C<list_repositories()>
+
+Returns a reference to a list with names for known repositories.
+
+TODO needs some kind of pseud-hash like object :)
+
+=cut
+
+sub list_repositories {
+	my $class = shift;
+	my (undef, $list) = _read_rep_list() or return ();
+	@$list = grep {! /^_/} @$list;
+	return $list;
+}
+
+=item C<set_default_repository(REPOSITORY)>
+
+Set the path or name for the default repository.
+
+=item C<get_default_repository()>
+
+Returns the path for the default repository, if there is one.
+
+=cut
+
+sub set_default_repository { $_[0]->set_repository(_default_ => $_[1]) }
+
+sub get_default_repository {
+	my $class = shift;
+	my $def = $class->get_repository('_default_') or return undef;
+	return $def if $def =~ /[\/\\]/;
+	return $class->get_repository($def);
+}
+
+=item C<set_repository(NAME => PATH)>
+
+Set the name for a repository.
+
+=item C<get_repository(NAME)>
 
 Returns a path for a repository with NAME or undef if the repository is unknown.
 The lookup is case in-sensitive.
 
 =cut
 
-sub lookup_by_name {
-	my ($class, $name) = @_;
-	$name = lc $name;
-	for (xdg_config_files('zim', 'repositories.list')) {
-		my ($hash) = Zim::File->new($_)->read_config;
-		my ($key) = grep {lc($_) eq $name} keys %$hash;
-		return Zim::File->abs_path($$hash{$key}, $ENV{HOME})
-			if defined $key;
-	}
-	return undef;
+sub set_repository {
+	my ($class, $name, $path) = @_;
+	my ($hash, $list) = _read_rep_list();
+	$hash ||= {};
+	$list ||= [];
+	die "TODO";
 }
 
+sub get_repository {
+	my ($class, $name) = @_;
+	my ($hash) = _read_rep_list() or return undef;
+	($name) = grep {lc($_) eq lc($name)} keys %$hash
+		unless exists $$hash{$name};
+	return $name ? $$hash{$name} : undef;
+}
+
+sub _read_rep_list {
+	my ($file) = xdg_config_files('zim', 'repositories.list');
+	return () unless defined $file;
+	return Zim::File->new($file)->read_config;
+}
+
+
 1;
 
 __END__
@@ -567,6 +646,6 @@ modify it under the same terms as Perl itself.
 =head1 SEE ALSO
 
 L<zim(1)>,
-L<Zim::Repository>
+L<Zim::Store>
 
 =cut
