@@ -1,16 +1,28 @@
 package Zim::File;
 
+use strict;
+our $VERSION = '0.20';
+
+eval 'use Zim::OS::Win32' if $^O eq 'MSWin32';
+die $@ if $@;
+
+our @ISA = ($^O eq 'MSWin32') ? ('Zim::File::Win32')
+                              : ('Zim::File::Unix' ) ;
+
+package Zim::File::Unix;
+
 use strict;
 use Carp;
 use File::Glob ':glob';
 use File::Spec;
 use File::Copy ();
+use File::BaseDir qw/xdg_cache_home/;
 use Encode;
 use Zim::Events;
 
 require Cwd;
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 our @ISA = qw/Zim::Events/;
 
 use overload
@@ -61,22 +73,6 @@ sub new {
 	bless {path => $class->abs_path($name)}, $class;
 }
 
-=item C<set_check_mtime(BOOLEAN)>
-
-Set whether to check mtime on read / write for this file.
-When turned on an exception is raised when the mtime has
-changed between read and write.
-
-=cut
-
-# feature turned off when undefined
-# when "0" turned on but no mtime set yet
-
-sub set_check_mtime {
-	$_[0]{mtime} = $_[1] ? ($_[0]{mtime} || 0) : undef;
-	$_[0]->signal_connect('write', \&_set_mtime) if $_[1];
-}
-
 =item C<path>
 
 Returns the full path to the file.
@@ -127,23 +123,46 @@ sub read {
 	return wantarray ? (@text) : join('', @text);
 }
 
-=item C<read_config()>
+=item C<read_config(\%CONFIG)>
 
 Read and parses contents.
 Returns a hash with key value mappings and a list which indicates the order
 in which the keys were found.
 
+The argument CONFIG is optional. If given this hash ref will be used to store
+the data.
+
 =cut
 
 sub read_config {
 	my $self = shift;
-	my (%config, @order);
+	my $config = shift || {};
+	my $order = [];
 	for ($self->read) {
-		/^\s*\#/ || ! /^\s*(.+?)=(.*)$/ and next;
-		$config{$1} = $2;
-		push @order, $1;
+		next if /^\s*\#/ || ! /^\s*(.+?)=(.*)$/;
+		$$config{$1} = $2 unless defined $$config{$1} and ! length $2;
+			# length 0 == undef
+		push @$order, $1;
 	}
-	return wantarray ? (\%config, \@order) : \%config;
+	return wantarray ? ($config, $order) : $config;
+}
+
+sub _read_config_alt {
+	# New ini-style format to be used for all new files
+	my $self = shift;
+	my $config = {};
+	my $group = $config;
+	for ($self->read) {
+		/^\s*\#/ and next;
+		if (/^\s*\[(.+)\]\s*$/) {
+			$$config{$1} ||= {};
+			$group = $$config{$1};
+		}
+		elsif (/^\s*(.+?)\s*=\s*(.+?)\s*$/) {
+			$$group{$1} = $2;
+		}
+	}
+	return $config;
 }
 
 =item C<read_hash(KEY, CASE)>
@@ -227,7 +246,7 @@ sub remove {
 	my $self = shift;
 	my $file = $self->{path};
 	unlink $file or die "Could not delete file: $file\n";
-	$self->{mtime} = 0 if defined $self->{mtime};
+	$self->{mtime} = 0;
 	$self->remove_dir;
 }
 
@@ -254,36 +273,57 @@ sub stat {
 	return { map {$stat[$_] => $val[$_]} 0 .. 12 };
 }
 
-=item C<open(MODE)>
+=item C<set_mtime()>
+
+Set the mtime for the file as an object attribute.
+
+=item C<check_mtime()>
+
+Get the mtime for the file and compare to stored mtime atribute.
+Dies when timestamps do not match. Used by subclasses to detect
+when files changed on disk.
+
+=cut
+
+sub set_mtime { $_[0]->{mtime} = (CORE::stat $_[0]->{path})[9] }
+
+sub check_mtime {
+	my $self = shift;
+	my $file = $self->{path};
+	my $mtime ||= (CORE::stat $file)[9];
+	die "File has changed on disk since reading: $file\n"
+		if  defined $mtime
+		and $self->{mtime} != $mtime ;
+	$self->{mtime} = $mtime;
+}
+
+=item C<open(MODE, FILE)>
 
 Returns an C<IO> object that can be used as a file handle.
 
 Make sure to close this handle using OO syntax, using C<< $fh->close >>
 instead of C<close $fh>.
 
+FILE is an optional argument allowing this function to be
+used without an object.
+
+MODE can be 'r', 'w' or 'a' for read, write and append respectivly.
+
 =cut
 
 sub open {
-	my ($self, $mode) = @_;
+	my ($self, $mode, $file) = @_;
+	my $cb = ! defined $file;
+	$file ||= $self->{path};
 	$mode ||= 'r';
 	#carp "## Zim::File::open($self, $mode)";
-	my $file = $self->{path};
-	if (defined $self->{mtime}) {
-		my $mtime = (CORE::stat $file)[9];
-		die "File has changed on disk since reading: $file\n"
-			if $mode eq 'w'
-			and $self->{mtime} and $self->{mtime} != $mtime ;
-		$self->{mtime} = $mtime; 
-	}
-	my $handle = IO::File::Zim->new($file, $mode) || die "$!: $file\n";
-	$handle->on_close(sub { $self->signal_emit('write', $file) })
-		if $mode eq 'w';
-	binmode $handle, ':utf8' unless $] < 5.008;
-	return $handle;
-}
+	#carp "Write $self" if $mode eq 'w';
 
-sub _set_mtime { # called by 'write' event
-	$_[0]{mtime} = (CORE::stat $_[0]{path})[9] if defined $_[0]{mtime};
+	my $handle = IO::File::Zim->new($file, $mode) || die "$!: $file\n";
+	$handle->set_attr(file => $self, mode => $mode) if $cb;
+	binmode $handle, ':utf8' unless $] < 5.008;
+	
+	return $handle;
 }
 
 =item C<grep(REGEXP, MODE)>
@@ -344,6 +384,26 @@ sub touch {
 	close FH;
 }
 
+=item C<commit_change()>
+
+If any kind of caching or versioning is done on the storage of this file,
+this method will commit all changes. Called automatically by DESTROY.
+
+File objects implementing this should check if there is any actual change
+before doing any heavy operation.
+
+=item C<discard_change()>
+
+If any kind of caching or versioning is done on the storage of this file,
+this method will discard all changes and reverse to the latest committed
+version.
+
+=cut
+
+sub commit_change {}
+
+sub discard_change {}
+
 =back
 
 =head2 Class Methods
@@ -443,12 +503,30 @@ Turns a path into an uri. Does no checks.
 =cut
 
 sub path2uri {
-	shift; # class
+	my $class = shift;
 	my $file = shift;
+	$file = $class->abs_path($file);
 	$file =~ s#^/?#file:///#;
 	return $file;
 }
 
+=item C<cache_path(@PATH)>
+
+Returns a path in the cache.
+
+=cut
+
+sub cache_path {
+	my $class = shift;
+	my @path = @_; # force copy
+	@path = map {
+		s/[\/\\:]+/_/g; # win32 save
+		s/^_+|_+$//g;
+		$_;
+	} @path;
+	return File::Spec->catfile(xdg_cache_home(), 'zim', @path);
+}
+
 =item C<localize(FILE)>
 
 Returns platform dependend version of FILE.
@@ -476,7 +554,7 @@ sub copy {
 	
 	$self->make_dir($self->dir($dest));
 	File::Copy::copy($source, $dest)
-		or die "Could not copy '$source' to '$dest'\n";
+		or die "Could not copy '$source' to '$dest' ($!)\n";
 }
 
 =item C<move(SOURCE, DEST)>
@@ -673,8 +751,183 @@ sub _grep_match {
 	return undef;
 }
 
+sub DESTROY { goto \&commit_change }
+
+
+
+=back
+
+=head1 SUBCLASSES
+
+=cut
+
+
+package Zim::File::CheckOnWrite;
+
+use strict;
+
+our @ISA = qw/Zim::File/;
+
+=head2 Zim::File::CheckOnWrite
+
+This subclass is used for the text files containing
+pages to prevent data loss.
+
+Be aware that checking and locking done in this class are only
+intended to prevent a single user to overwrite data by accident.
+It will not suffice for a multi-user system.
+
+This class checks the modification time of the file when
+reading and writing. It will not allow you to overwrite a file
+if it changed since you read it.
+
+Also this class implements "atomic" writing. This means that
+when you write to a file you write to a temporary file first.
+When the file handle is closed the temporary file is moved to
+replace the original file. This way the original file is not
+purged when an error occurs while writing.
+
+=cut
+
+sub new {
+	my $class = shift;
+	my $self = $class->SUPER::new(@_);
+	$self->{mtime} = 0;
+	$self->{tmp_path} = $self->{path} . '.new~';
+	$self->signal_connect('write', \&on_write);
+	return $self;
+}
+
+sub on_write {
+	# rename tmp file to real path and record mtime
+	my $self = shift;
+	#warn "move $self->{tmp_path} => $self->{path}\n";
+	File::Copy::move($self->{tmp_path} => $self->{path})
+		or die "Could not move '$$self{tmp_path}' to '$$self{path}'\n";
+	$self->set_mtime;
+}
+
+sub open {
+	my ($self, $mode) = @_; 
+	$mode ||= 'r';
+	
+	($mode eq 'r') ? $self->set_mtime : $self->check_mtime ; # mode 'w' || 'a'
+
+	if ($mode eq 'r') {
+		return $self->SUPER::open($mode);
+	}
+	else { # mode 'w' || 'a'
+		# open tmp file but connect callback for write
+		my $handle = $self->SUPER::open($mode, $self->{tmp_path});
+		$handle->set_attr(file => $self, mode => $mode);
+		return $handle;
+	}
+}
+
+
+
+package Zim::File::CacheOnWrite;
+
+use strict;
+
+our @ISA = qw/Zim::File/;
+
+=head2 Zim::File::CacheOnWrite
+
+This subclass is used when the original file is located on a slow
+file system. It implements a "copy on write" that uses a cache file
+after the first write. You need to call C<commit_change()> in order
+to overwrite the original file.
+
+It does check the modification time like CheckOnWrite.
+
+=over 4
+
+=item C<set_cache(PATH)>
+
+Set the file name to use as cache.
+
+=cut
+
+sub new {
+	my $class = shift;
+	my $self = $class->SUPER::new(@_);
+	$self->{mtime} = 0;
+	$self->{cache_path} = $self->cache_path($self->{path});
+	#warn "path: $self->{path}\ncache_path: $self->{cache_path}\n";
+	if (-f $self->{cache_path}) { # file exists - get original mtime
+			my $mtime = Zim::File->new($self->{cache_path}.'.mtime')->read();
+			chomp $mtime;
+			$self->{mtime} = $mtime || 0;
+	}
+	$self->signal_connect('write', \&on_write);
+	return $self;
+}
+
+sub on_write { $_[0]->set_mtime }
+
+sub open {
+	my ($self, $mode) = @_;
+	$mode ||= 'r';
+
+	my $cache = $self->{cache_path};
+	die "BUG: no cache path set" unless $cache;
+	#warn "open $mode cache: $cache, exists: ".(-f $cache)."\n";
+	
+	if ($mode eq 'r' and ! -f $cache) {
+		$self->set_mtime;
+		return $self->SUPER::open($mode);
+	}
+	# else cache exists or mode 'w' or 'a'
+	
+	$self->make_dir( $self->dir($cache) ) if $mode eq 'w';
+	Zim::File->new($cache.'.mtime')->write($self->{mtime});
+	return Zim::File->open($mode, $cache);
+		# no "write" event call back with this syntax
+		# therefore no mtime hook here
+}
+
+=item C<commit_change()>
+
+Copy the cached data back to the original file.
+
+=cut
+
+sub commit_change {
+	my $self = shift;
+	return unless -f $self->{cache_path};
+	
+	$self->check_mtime;
+
+	warn "## move $self->{path} => $self->{cache_path}\n";
+	my $out = $self->SUPER::open('w');
+	my $in = Zim::File->open('r', $self->{cache_path});
+	File::Copy::copy($in => $out)
+		or die "Could not copy '$$self{cache_path}' to '$$self{path}'\n";
+	$in->close;
+	$out->close; # will trigger "write" event
+
+	$self->discard_change();
+}
+
+=item C<discard_change()>
+
+Throw away the cached version.
+
+=cut
+
+sub discard_change {
+	return unless -f $_[0]->{cache_path};
+	unlink $_[0]->{cache_path}
+		or die "$!: $_[0]->{cache_path}\n";
+	unlink $_[0]->{cache_path}.'.mtime';
+}
+
+
+
 package IO::File::Zim;
 
+use strict;
 use IO::File;
 
 our @ISA = qw/IO::File/;
@@ -691,9 +944,9 @@ our @ISA = qw/IO::File/;
 
 our %_objects;
 
-*on_close = sub { # used to define a callback
+*set_attr = sub { # used to set object attributes
 	my $fh = shift;
-	$_objects{"$fh"}{on_close} = shift;
+	$_objects{"$fh"} = {@_};
 };
 
 *close = sub {
@@ -701,10 +954,20 @@ our %_objects;
 	my $re = $fh->SUPER::close(@_);
 	return $re unless exists $_objects{"$fh"};
 	my $obj = delete $_objects{"$fh"};
-	$obj->{on_close}->() if $obj->{on_close};
+	$$obj{file}->signal_emit('write') if $$obj{mode} eq 'w';
 	return $re;
 };
 
+*DESTROY = sub {
+	my $fh = shift;
+	return unless exists $_objects{"$fh"};
+	my $obj = delete $_objects{"$fh"};
+	return unless $$obj{mode} eq 'w';
+	warn "File handle not closed after writing for:\n\t$$obj{file}\n";
+	# We do not trigger the signal here - close might not have
+	# happened due to an error while writing atomically.
+};
+
 1;
 
 __END__
