@@ -4,7 +4,7 @@ use strict;
 no warnings;
 use Zim::Formats;
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 our @ISA = qw/Zim::Formats/;
 
 # TODO: some tags can be nested: email links for example
@@ -131,8 +131,9 @@ sub parse_links {
 sub _parse_link {
 	my $l = shift;
 	my $text;
-	if ($l =~ /^\[\[([^|]+)\|?(.*)\]\]$/) { # [[link]] or [[link|text]]
+	if ($l =~ /^\[\[([^|]*)\|?(.*)\]\]$/) { # [[link]] or [[link|text]]
 		($l, $text) = ($1, $2);
+		$l = $text unless length $l; # [[|link]] bug
 	}
 	if ($l =~ /^mailto:|^\S+\@\S+\.\w+$/) {
 		$text = $l unless length $text;
@@ -144,14 +145,15 @@ sub _parse_link {
 sub parse_urls {
 	my ($class, $text) = @_;
 	my $i = 0;
+	my $C = q/[^\s\"\<\>\']/; # limit the character class a bit
 	return
 		map   { ($i++ % 2) ? _parse_link($_) : $_ }
 		split m#(
-			\b\w[\w\+\-\.]+://  \S*\[\S+\](?:\S+[\w\/])? |
-			\b\w[\w\+\-\.]+://  \S+[\w\/]                |
-			\bmailto:\S+\@      \S*\[\S+\](?:\S+[\w/])?  |
-			\bmailto:\S+\@      \S+[\w/]                 |
-			\b\S+\@\S+\.\w+\b
+			\b\w[\w\+\-\.]+://  $C*\[$C+\](?:$C+[\w\/])? |
+			\b\w[\w\+\-\.]+://  $C+[\w\/]                |
+			\bmailto:$C+\@      $C*\[$C+\](?:$C+[\w/])?  |
+			\bmailto:$C+\@      $C+[\w/]                 |
+			\b$C+\@$C+\.\w+\b
 		)#x, $text;
 		# The host name in an uri can be "[hex:hex:..]" for ipv6
 		# but we do not want to match "[http://foo.org]"
