@@ -13,6 +13,7 @@ use Gtk2::Gdk::Keysyms;
 use Gtk2::SimpleList;
 use Zim;
 use Zim::Events;
+use Zim::Utils;
 use Zim::GUI::Component;
 use Zim::GUI::PathBar;
 use Zim::GUI::PageView;
@@ -22,7 +23,7 @@ my $has_mimeapplications = $@ ? 0 : 1;
 warn "WARNING: Could not use 'File::MimeInfo::Applications', disabling application bindings\n"
 	unless $has_mimeapplications;
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 our @ISA = qw/Zim::Events Zim::GUI::Component/;
 
 =head1 NAME
@@ -73,13 +74,11 @@ width		500
 height		350
 x
 y
-default_root
 default_home	:Home
 default_type	Files
 browser
 file_browser
 email_client
-date_format	%A %d/%m/%Y
 undo_max	50
 follow_new_link	0
 backsp_unindent	1
@@ -98,16 +97,22 @@ use_autoselect	1
 follow_on_enter	1
 use_ctrl_space	1
 ro_cursor	0
+expand_tree	0
+save_interval	5000
 };
 
-our %DEFAULTS = (
-	file_browser => q/rox '%d'/,
-	browser	     => q/firefox '%u'/,
-	email_client => q/thunderbird '%u'/,
-);
+our %DEFAULTS;
+for (
+	[file_browser => q/rox '%d'/],
+	[browser	     => q/firefox '%u'/],
+	[email_client => q/thunderbird '%u'/],
+) {
+	$DEFAULTS{$$_[0]} ||= $$_[1];
+}
 
 my $ui_menus =
 # name		stock id 	label
+## __actions
 q{
 FileMenu	.		_File
 EditMenu	.		_Edit
@@ -119,9 +124,11 @@ GoMenu		.		_Go
 HelpMenu	.		_Help
 PathBarMenu	.		P_athbar type
 };
+##
 
 my $ui_actions = 
 # name,		stock id,	label,		accelerator,	tooltip
+## __actions
 q{
 NewPage		gtk-new		_New Page	.		New page
 popup_NewPage	gtk-new		_New Page	.		New page
@@ -154,26 +161,33 @@ GoToday		.		To_day		<alt>D		Today
 GoHome		gtk-home	_Home		<alt>Home	Go home
 JumpTo		gtk-jump-to	_Jump To...	<ctrl>J		Jump to page
 ShowHelp	gtk-help	_Contents	F1		Help contents
+ShowHelpKeys	.		_Keybindings	.		Key bindings
+ShowHelpBugs	.		_Bugs		.		Bugs
 About		gtk-about	_About		.		About
 };
+##
 
 my $ui_toggle_actions =
 # name,		stock id,	label,		accelerator,	tooltip 
+## __actions
 q{
 TToolBar	.		Toolbar		.		Show toolbar
 TStatusBar	.		_Statusbar	.		Show statusbar
 TPane		gtk-index	Side _Pane	F9		Show side pane
 TCalendar	stock_calendar-view-month	Calen_dar	<ctrl><shift>D	Show calendar
 };
+##
 
 my $ui_radio_actions =
 # name,		stock id,	label,		accelerator,	tooltip
+## __actions
 q{
 PBRecent	.		_Recent pages	.		.
 PBHistory	.		_History	.		.
 PBNamespace	.		_Namespace	.		.
 PBHidden	.		H_idden		.		.
 };
+##
 
 my $ui_layout = q{<ui>
 	<menubar name='MenuBar'>
@@ -188,7 +202,7 @@ my $ui_layout = q{<ui>
 			<placeholder name='PrintActions'/>
 			<menuitem action='EmailPage'/>
 			<separator/>
-			<placeholder name='FileMods'/>
+			<placeholder name='PageMods'/>
 			<separator/>
 			<menuitem action='Props'/>
 			<separator/>
@@ -251,6 +265,8 @@ my $ui_layout = q{<ui>
 		</menu>
 		<menu action='HelpMenu'>
 			<menuitem action='ShowHelp'/>
+			<menuitem action='ShowHelpKeys'/>
+			<menuitem action='ShowHelpBugs'/>
 			<menuitem action='About'/>
 		</menu>
 	</menubar>
@@ -287,7 +303,7 @@ my $ui_layout = q{<ui>
 my $ui_layout_rw = q{<ui>
 <menubar name='MenuBar'>
 		<menu action='FileMenu'>
-			<placeholder name='FileMods'>
+			<placeholder name='PageMods'>
 				<menuitem action='RenamePage'/>
 				<menuitem action='DeletePage'/>
 			</placeholder>
@@ -307,8 +323,6 @@ sub init {
 	my $self = shift;
 	# Initialize attributes
 	$self->{app} = $self; # fool base class
-	$self->{icon_file} ||= $Zim::ICON; # defined in Component.pm
-	$self->{profile}   ||= 'default';
 	$self->{_message_timeout} = -1;
 	$self->{_save_timeout}    = -1;
 	
@@ -320,6 +334,7 @@ sub init {
 		);
 	$self->{history} = $repository->init_history();
 	my $dir = $repository->{dir}; # FIXME what if there is no dir
+
 	$self->{name} ||= $repository->config->{name};
 	unless ($self->{name}) { # needed for trayicon tooltip
 		my @dirs = grep length($_),
@@ -329,6 +344,8 @@ sub init {
 	$repository->config->{name} = $self->{name};
 	warn "# Repository name: $self->{name}\n";
 
+	$self->{profile} ||= $repository->config->{profile} || 'default';
+
 	if (my $icon = $repository->config->{icon}) {
 		my $file = File::Spec->rel2abs($icon, $dir);
 		($file) = xdg_data_files('pixmaps', $icon) unless -f $file;
@@ -336,19 +353,10 @@ sub init {
 	}
 
 	$self->{repository} = $repository;
-	$repository->{date_format} = $self->{settings}{date_format};
 	$repository->{user_name} = $self->{settings}{user_name};
 
 	# Load config
 	$self->load_config;
-	for (split /\n/, $CONFIG) { # load defaults
-		next unless /\S/;
-		my ($key, $val) = split /\t/, $_, 2;
-		$$self{settings}{$key} = $val
-			unless defined $$self{settings}{$key};
-	}
-	$$self{settings}{user_name} ||= $ENV{USER};
-	
 	$self->set_home();
 
 	return $self;
@@ -362,15 +370,22 @@ This method initializes all GUI objects that make up the application.
 
 sub gui_init {
 	my $self = shift;
-	$self->_load_icons;	
 
 	## Setup the window
 	my $window = Gtk2::Window->new('toplevel');
 	$window->set_default_size(@{$self->{settings}}{'width', 'height'});
 	$window->signal_connect_swapped(delete_event => \&on_delete_event, $self);
 	$window->signal_connect(destroy => sub { Gtk2->main_quit });
-	$window->set_icon(
-		Gtk2::Gdk::Pixbuf->new_from_file($self->{icon_file}) );
+#	if (	defined $self->{icon_file}
+#		or ! Gtk2->CHECK_VERSION(2, 6, 0)
+#	) {
+		$window->set_icon(
+			Gtk2::Gdk::Pixbuf->new_from_file(
+				$self->{icon_file} || $Zim::ICON) );
+#	}
+#	else {
+#		$window->set_icon_name('zim'); # since 2.6.0
+#	}
 	$window->set_title('Zim');
 	$self->{window} = $window;
 
@@ -411,7 +426,6 @@ sub gui_init {
 		GoBack    => 0,
 		GoForward => 0,
 		GoParent  => 0,
-		EmailPage => 0,
 	);
 	
 	$self->add_ui($ui_layout);
@@ -528,11 +542,6 @@ sub gui_init {
 			GoParent   => ($self->{page}->namespace =~ /[^:]/),
 		)
 	} ) if $hist;
-	$self->signal_connect('page_loaded', sub {
-		my $self = shift;
-		$self->actions_set_sensitive(
-			EmailPage => $self->{page}->has_source );
-	} );
 
 	## List autoloaded components
 	$self->{autoload}{$_} = 'Zim::GUI::'.$_
@@ -557,28 +566,6 @@ sub gui_init {
 #		} );
 }
 
-sub _load_icons { # load all icons in pixmaps/zim
-	my @dirs = grep -d $_, map "$_/pixmaps/zim",
-		xdg_data_home, xdg_data_dirs;
-	my @files;
-	for my $d (@dirs) {
-		push @files, map $d.'/'.$_, Zim::File->list_dir($d);
-	}
-	return unless @files;
-
-	my $factory = Gtk2::IconFactory->new;
-	$factory->add_default;
-	for my $f (@files) {
-		my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($f);
-		my $icon_set = Gtk2::IconSet->new_from_pixbuf($pixbuf);
-		my $n = $f;
-		$n =~ s/.*[\/\\]//;
-		$n =~ s/\..*//;
-		#warn "Icon: $n => $f\n";
-		$factory->add($n => $icon_set);
-	}
-}
-
 =item C<gui_show()>
 
 =cut
@@ -622,9 +609,10 @@ sub gui_show {
 	$self->TPathBar($pbtype);
 	$self->{message_lock} = 0;
 
-	## set save interval
+	## set autosave interval - default is every 5 seconds
+	my $t = $$set{save_interval} || 5000;
 	$self->{_save_timeout} = 
-		Glib::Timeout->add(5000, sub {$self->save_page; 1})
+		Glib::Timeout->add($t, sub {$self->SaveIfModified('SOFT'); 1})
 		unless $self->{settings}{read_only};
 	
 	$self->signal_emit('gui_show');
@@ -729,8 +717,7 @@ sub on_delete_event {
 		return 1;
 	}
 	
-	$self->{save_lock} = 0; # just to be sure
-	$self->save_page() || return 1;
+	$self->SaveIfModified() || return 1;
 	$self->save_config;
 
 	$self->{history}->write if $self->{history};
@@ -868,17 +855,22 @@ sub load_config {
 	my $self = shift;
 	my $p = $self->{profile};
 
+	# load defaults
+	for (split /\n/, $CONFIG) {
+		next unless /\S/;
+		my ($key, $val) = split /\s+/, $_, 2;
+		$$self{settings}{$key} = $val
+	}
+	$$self{settings}{user_name} ||= $ENV{USER};
+
+	# load file
 	my $file = _xdg_config_data_files('zim', "$p.conf");
 	$file = _xdg_config_data_files('zim', 'default.conf') unless $file;
 	return warn "WARNING: Could find default configuration\n" unless $file;
-	warn "# Using config from: $file\n";
-	
-	my $conf = $file->read_config();
-	for (keys %$conf) {
-		next unless length $$conf{$_};
-		$self->{settings}{$_} = $$conf{$_};
-	}
+	warn "# Profile: $p\n# Using config from: $file\n";
+	$file->read_config($self->{settings});
 
+	# load accelerators
 	my $accelmap = _xdg_config_data_files('zim', 'accelmap');
 	Gtk2::AccelMap->load($accelmap) if $accelmap;
 }
@@ -905,9 +897,9 @@ sub save_config {
 	$self->{settings}{pane_pos} = $self->{hpaned}->get_position;
 	my $p = $self->{profile};
 	my %conf = %{$self->{settings}};
-	Zim::File->new(
-		xdg_config_home, 'zim', "$p.conf"
-	)->write_config(\%conf);
+	my $file = Zim::File->new(xdg_config_home, 'zim', "$p.conf");
+	warn "# Writing config to: $file\n";
+	$file->write_config(\%conf);
 
 	my $accelmap = File::Spec->catfile(
 		xdg_config_home, 'zim', 'accelmap' );
@@ -954,13 +946,15 @@ Calls C<open_directory()> when FILE is a directory.
 
 sub open_file {
 	my ($self, $file) = @_;
-	return $self->open_directory($file) if $file =~ m#/$# or -d $file;
+	return $self->open_directory($file) if $file =~ /\/$/ or -d $file;
 	my $mt = mimetype($file);
 	
 	return $self->open_directory($file, 'OPEN_FILE')
 		unless $mt and $has_mimeapplications;
 	
-	my ($app) = mime_applications($mt);
+	my ($app) = grep $_, mime_applications($mt);
+		# if no default, select first
+		# FIXME: ask user for default
 
 	return $self->open_directory($file, 'OPEN_FILE') unless $app;
 
@@ -1004,7 +998,7 @@ sub open_directory {
 	
 	$dir = Zim::File->localize($dir);
 	$browser =~ s/\%[sfd]/$dir/ or $browser .= " \"$dir\"";
-	$self->_exec($browser);
+	Zim::Utils->run($browser);
 }
 
 =item C<open_url(URL)>
@@ -1060,7 +1054,7 @@ sub open_url {
 	}
 	
 	$browser =~ s/\%[us]/$url/ or $browser .= " \"$url\"";
-	$self->_exec($browser);
+	Zim::Utils->run($browser);
 }
 
 =item C<load_page(PAGE)>
@@ -1074,11 +1068,9 @@ use C<link_clicked()> for relative page names, urls etc.
 
 sub load_page {
 	my ($self, $page) = @_;
-	
-	if ($self->{page}) {
-		$self->{save_lock} = 0; # just to be sure
-		$self->save_page || return;
-	}
+
+	$self->SaveIfModified or return
+		if $self->{page};
 
 	goto &_load_page;
 }
@@ -1212,73 +1204,77 @@ sub OpenRep {
 		$self->exec_new_window($key);
 	}
 	else {
-		my $file = Zim->lookup_by_name($key);
+		my $dir = Zim->get_repository($key);
 		return $self->error_dialog("No such repository: $key")
-			unless defined $file;
-		$self->exec_new_window($file);
+			unless defined $dir;
+		$self->exec_new_window($dir);
 	}
 }
 
 =item C<Save()>
 
-Force saving the current page.
+Force saving the current page and state.
+Returns boolean for success.
+
+=item C<SaveIfModified(SOFT)>
+
+Save the page if it was modified.
+Returns boolean for success or if page was not changed.
+
+SOFT is an optional argument. If this argument is true the page may not be
+saved if there is an issue. This is used to skip auto-saving in some cases.
+Also "soft" changes may not be committed for version management etc.
 
 =cut
 
 sub Save {
 	my $self = shift;
-	$self->save_page('FORCE');
+	warn "## GUI::Save\n";
+	$self->_save_page(1, 1) or return 0;
 	$self->save_config;
+	return 1;
 }
 
-=item C<save_page(FORCE)>
-
-Check if the current page needs to be saved.
-If FORCE is true the page is saved whether it was modified or not.
-
-=cut
-
-# TODO split out to save_page_if_modified() and save_page()
-# save_page_if_modified(FORCE) overrules save_lock
-# save_page() just saves it, no questions asked
-# (maybe call the first one check_save_page() ?)
+sub SaveIfModified {
+	my ($self, $soft) = @_;
+	return 1 if $self->{settings}{read_only}; # not modified, so save is ok
+	return 0 if $self->{save_lock} and $soft;
+	
+	my $modified = $self->{objects}{PageView}->modified;
+	
+	warn "## GUI::SaveIfModified\n";
+	return $self->_save_page($modified, !$soft);
+}
 
 # TODO part of this logic belongs in the PageView component
 
-sub save_page {
-	my ($self, $force) = @_;
-	return 1 if $self->{app}{settings}{read_only}; # not modified, so save is ok
-	my $modified = $self->{objects}{PageView}->modified;
-	unless ($force) {
-		return 1 if ! $modified;
-		return 0 if $self->{save_lock};
-	}
-	warn "## GUI::save_page force: $force modified: $modified\n";
-	
+sub _save_page {
+	my ($self, $save, $commit) = @_;
 	$self->{save_lock} = 1; # Used to block autosave while "could not save" dialog
 	
 	my $page = $self->{page};
-	eval { $self->PageView->save_page($page); 1 };
-	unless ($@) {
-		$self->PageView->modified(0);
-		$self->signal_emit('page_saved', $page->name);
-	}
-	else {
+	eval {
+		$self->PageView->save_page($page) if $save;
+		$page->commit_change() if $commit;
+	};
+	if ($@) {
 		my $error = $@;
 		warn 'ERROR: '.$error;
 		my $answer = $self->prompt_question(
 			'Could not save - Zim', 'error',
 			"<b>Could not save page</b>\n\n$error",
-			['discard', undef, '_Discard changes'],
+			['cancel', 'gtk-cancel', '_Cancel' ],
+			['discard', 'gtk-delete', '_Discard changes'],
 			['copy', 'gtk-save-as', '_Save a copy...' ],
-			2 );
+			3 );
 		if ($answer eq 'discard') {
+			$self->{page}->discard_change;
 			$self->_load_page($self->{page}); # load page wihtout saving
 		}
 		elsif ($answer eq 'copy') {
-			return $self->SaveCopy; # recurs inderectly
+			return $self->SaveCopy; # recurs indirectly
 		}
-		else { return 0 }
+		else { return 0 } # return _without_ removing save_lock
 	}
 
 	$self->update_status();
@@ -1323,7 +1319,7 @@ sub SaveCopy {
 	# change page and save buffer
 	$self->{page} = $page;
 	$self->Save;
-	$self->load_page($page->name); # trick into reloading
+	$self->Reload;
 }
 
 =item C<Export()>
@@ -1340,9 +1336,9 @@ Open the current page in the email client.
 
 sub EmailPage {
 	my $self = shift;
-	$self->Save or return;
+	$self->SaveIfModified or return;
 	my $subject = $self->{page}->name;
-	my $io = $self->{page}->open_source('r');
+	my $io = $self->{page}->get_text;
 	my $src = join '', <$io>;
 	$src =~ s/(\W)/sprintf '%%%02x', ord $1/eg; # url encoding
 	$self->open_url("mailto:?subject=$subject&body=".$src);
@@ -1350,7 +1346,7 @@ sub EmailPage {
 
 =item C<RenamePage(FROM, TO, UPDATE_SELF, UPDATE_OTHER)>
 
-Wrapper for C<< Zim::Repository->move_page() >>.
+Wrapper for C<< Zim->move_page() >>.
 
 Move page from FROM to TO. If TO is undefined a dialog is shown to ask for a
 page name.
@@ -1370,11 +1366,8 @@ sub RenamePage {
 	$from = $self->{page}->name unless defined $from;
 	
 	my $move_current = $self->{page}->equals($from);
-	if ($move_current) { # FIXME clearer interface for this
-		$self->{save_lock} = 0 if $move_current;
-		$self->save_page or return 0;
-	}
-	else { $self->save_page }
+	my $save = $self->SaveIfModified(!$move_current);
+	return 0 if $move_current and !$save;
 	
 	unless (defined $to) {
 		($to, @update) = $self->prompt_rename_page_dialog($from);
@@ -1441,32 +1434,36 @@ sub RenamePage {
 	return 1;
 }
 
-=item C<DeletePage(PAGE)>
+=item C<DeletePage(PAGE, RECURS, NO_CONFIRM)>
 
-Wrapper for C<< Zim::Repository->delete_page >>.
+Wrapper for C<< Zim->delete_page >>.
 Asks the user for confirmation.
 
 If PAGE is undefined the current page is deleted.
 
+TODO: make recusive delete work
+
 =cut
 
 sub DeletePage {
 	# FIXME option to delete a complete sub-tree
 	# FIXME integrate with the save_page() logic, now we have 
 	# redundant code here
-	my ($self, $page) = @_;
+	my ($self, $page, undef, $noconf) = @_;
 	$page = $self->{page}->name unless defined $page;
 	
 	my $name = ref($page) ? $page->name : $page;
 	my $delete_current = ($name eq $self->{page}->name);
-	$self->save_page; # Mind you, overriding save lock could lock us here
+	$self->SaveIfModified(!$delete_current);
 	
-	my $answer = $self->prompt_question(
-		'Delete page - Zim', 'warning',
-		"Are you sure you want to\ndelete '$name' ?",
-		['no',  'gtk-cancel', undef],
-		['yes', 'gtk-delete', undef]  );
-	return unless $answer eq 'yes';
+	unless ($noconf) {
+		my $r = $self->prompt_question(
+			'Delete page - Zim', 'warning',
+			"Are you sure you want to\ndelete '$name' ?",
+			['no',  'gtk-cancel', undef],
+			['yes', 'gtk-delete', undef]  );
+		return unless $r eq 'yes';
+	}
 	
 	$page = $self->{page} if $delete_current; # make sure we have the right object
 	eval { $self->{repository}->delete_page($page) };
@@ -1841,15 +1838,30 @@ sub JumpTo {
 		($page) = @$values;
 		return unless $page =~ /\S/;
 	}
-
-	return $self->load_page($page) if ref $page;
+	elsif (ref $page) {
+		return $self->load_page($page);
+	}
 
 	$page = $self->check_page_input($page);
 	return unless defined $page;
+
 	$self->load_page(
 		$self->{repository}->resolve_name($page, $self->{page}) );
 }
 
+=item C<ShowHelpKeys()>
+
+=item C<ShowHelpBugs()>
+
+Shows help on keybindings and bugs. Trying to seduce people to actually read
+the manual...
+
+=cut
+
+sub ShowHelpKeys { $_[0]->ShowHelp(':zim:usage:keybindings') }
+
+sub ShowHelpBugs { $_[0]->ShowHelp(':zim:bugs') }
+
 =item C<About()>
 
 This dialog tells you about the version of zim you are using.
