@@ -7,7 +7,7 @@ use Zim::Selection;
 use Zim::GUI::Component;
 #use Zim::Formats;
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 our @ISA = qw/Zim::GUI::Component/;
 
 =head1 NAME
@@ -32,7 +32,7 @@ Present the dialog.
 sub show {
 	my $self = shift;
 	return $self->{dialog}->present if defined $self->{dialog};
-	$self->{app}->save_page || return;
+	$self->{app}->SaveIfModified or return;
 
 	my $dialog = Gtk2::Dialog->new(
 		"Export page - Zim", $self->{app}{window},
@@ -95,7 +95,7 @@ sub show {
 	$frame->add($table);
 	
 	my $i;
-	for (qw/Format Template/, '', 'Output dir') {
+	for (qw/Format Template/, '', 'Output dir', 'Index page') {
 		if (length $_) {
 			my $label = Gtk2::Label->new($_.':');
 			my $align = Gtk2::Alignment->new(0,0.5, 0,0);
@@ -141,11 +141,16 @@ sub show {
 	} );
 	$table->attach_defaults($button3, 2,3, 3,4);
 	
+	my $entry4 = Gtk2::Entry->new;
+	$table->attach_defaults($entry4, 1,2, 4,5);
+	$entry4->set_text( $$conf{export_index} || 'index' );
+
 	my $fill_combo2 = sub {
 		my $format = _get_active_text($combo1);
 		for (reverse 0 .. $combo2_items) { $combo2->remove_text($_) }
 		$templates = Zim::Formats->list_templates($format);
 		for (sort keys %$templates) {
+			next if /^[_\W]/; # skip "_New" etc.
 			if (/^default$/i) { $combo2->prepend_text($_) }
 			else              { $combo2->append_text($_)  }
 		}
@@ -237,9 +242,10 @@ sub show {
 			dir      => $entry3->get_text,
 			template => $template,
 			media    => $media,
+			index    => $entry4->get_text
 		);
-		@$conf{qw/export_dir export_format export_media/}
-			= @opts{qw/dir format media/};
+		$$conf{'export_'.$_} = $opts{$_}
+			for qw/dir format media index/;
 		$$conf{'export_template'} = $tn;
 		$dialog->destroy;
 		$self->{dialog} = undef;
@@ -264,7 +270,7 @@ sub show {
 		};
 		$self->error_dialog($@) if $@;
 
-		$self->{app}{repository}->save_config(); # save settings
+		$self->{app}{repository}->save(); # save settings
 		
 		last;
 	}
