@@ -7,7 +7,7 @@ use overload
 use Carp;
 use Zim::Formats;
 
-our $VERSION = '0.18';
+our $VERSION = '0.20';
 
 our %_Formats;
 
@@ -17,8 +17,8 @@ Zim::Page - Page object for Zim
 
 =head1 DESCRIPTION
 
-This class defines a page object. This is a data container used by
-L<Zim::Repository> to represent for example a file.
+This class defines a page object. A page is the unit of data as presented
+to the user. The source of the data is managed by L<Zim::Store> objects.
 
 =head1 OVERLOAD
 
@@ -44,7 +44,8 @@ sub new {
 	$name =~ s/^:?/:/;
 	my $self = bless {
 		name => $name,
-		repository => $parent,
+		store => $parent,
+		root => $parent ? $parent->root : undef,
 		status => '',
 		properties => {read_only => 1},
 	}, $class ;
@@ -136,7 +137,7 @@ Returns TRUE if the page already exists.
 
 =cut
 
-sub exists { $_[0]->{status} ne 'new' and $_[0]->{status} ne 'deleted' }
+sub exists { $_[0]->{source} ? $_[0]->{source}->exists : $_[0]->{exists} }
 
 =item C<copy(TARGET, UPDATE_LINKS)>
 
@@ -146,7 +147,7 @@ sub exists { $_[0]->{status} ne 'new' and $_[0]->{status} ne 'deleted' }
 
 The methods C<copy()>, C<move()> and C<delete()> are aliases for the methods
 C<copy_page()>, C<move_page()> and C<delete_page()> in the public repository
-interface; see L<Zim::Repository>.
+interface; see L<Zim::Store>.
 
 TARGET is the new name for the page. The UPDATE_LINKS argument is a boolean
 that tells whether links to this page should be updated to point to this
@@ -154,14 +155,14 @@ new name.
 
 =cut
 
-sub copy { $_[0]->{repository}->root->copy_page(@_) }
+sub copy { $_[0]->{root}->copy_page(@_) }
 
 sub move {
 	$_[0]->{_resolve} = {}; # clear links - FIXME update here ?
-	$_[0]->{repository}->root->move_page(@_);
+	$_[0]->{root}->move_page(@_);
 }
 
-sub delete { $_[0]->{repository}->delete_page(@_) }
+sub delete { $_[0]->{store}->delete_page(@_) }
 
 =item C<parse_link(LINK)>
 
@@ -189,7 +190,7 @@ Caching wrapper for C<$repository->resolve_name(NAME, PAGE, NO_DEFAULT)>.
 sub resolve_page {
 	my $self = shift;
 	my $name = $self->resolve_name(@_);
-	return $name ? $self->{repository}->root->get_page($name) : undef ;
+	return $name ? $self->{root}->get_page($name) : undef ;
 }
 
 sub resolve_name { # 'cloning' hack needed when exporting
@@ -202,8 +203,7 @@ sub resolve_name { # 'cloning' hack needed when exporting
 		return ( ($no_def && $$r[1]) ? undef : $$r[0] )
 	}
 
-	my $root = $self->{repository}->root;
-	my $name = $root->resolve_name($link, $self, 1);
+	my $name = $self->{root}->resolve_name($link, $self, 1);
 	if ($name) {
 		$self->{_resolve}{$link} = [$name, 0];
 	}
@@ -223,8 +223,8 @@ TODO: stable api for this
 
 sub match_word { # TODO optimize by caching found links
 	my ($self, $word) = @_;
-	return $self->{repository}->can('_match_word')
-		? $self->{repository}->_match_word($self, $word)
+	return $self->{store}->can('_match_word')
+		? $self->{store}->_match_word($self, $word)
 		: undef ;
 }
 
@@ -238,13 +238,14 @@ This does not guarantee that the actual content is the same.
 sub equals {
 	my ($self, $page) = @_;
 	return $page eq $self->{name} unless ref $page; 
-	return 0 if $page->{repository} ne $self->{repository};
+	return 0 if $page->{store} ne $self->{store}; # compare ref string
 	return $self->{name} eq $page->name;
 }
 
 =item C<clone(SOURCE)>
 
 Import content from object SOURCE into this object.
+Deletes current contents.
 
 =cut
 
@@ -252,6 +253,7 @@ sub clone {
 	my ($self, $page, %opt) = @_;
 	local $self->{cloning} = $page;
 	if ( $self->{format} and $page->{format} ) {
+		$self->delete if $self->exists; # prevent mtime check etc.
 		my $tree = $page->get_parse_tree;
 		$self->_check_media($tree, $opt{media}, $page)
 			if $opt{media} and $opt{media} ne 'relative';
@@ -287,8 +289,8 @@ sub _copy_media {
 	my ($self, $file, $cloning) = @_;
 
 	my $copy;
-	my $croot = $cloning->{repository}->root->{dir};
-	my $root = $self->{repository}->root->{dir};
+	my $croot = $cloning->{root}{dir};
+	my $root = $self->{root}{dir};
 	my $rel = Zim::File->rel_path($file, $croot);
 	if (defined $rel) { # was relative
 		$copy = Zim::File->abs_path($rel, $root);
@@ -324,9 +326,9 @@ sub get_next {
 	my $self = shift;
 	my ($ns, $name) = ($self->namespace, $self->basename);
 	my $prev;
-	for ($self->{repository}->list_pages($ns)) {
+	for ($self->{store}->list_pages($ns)) {
 		s/:$//;
-		return $self->{repository}->get_page($ns.$_)
+		return $self->{store}->get_page($ns.$_)
 			if $prev eq $name;
 		$prev = $_;
 	}
@@ -337,10 +339,10 @@ sub get_prev {
 	my $self = shift;
 	my ($ns, $name) = ($self->namespace, $self->basename);
 	my $prev;
-	for ($self->{repository}->list_pages($ns)) {
+	for ($self->{store}->list_pages($ns)) {
 		s/:$//;
 		if ($_ eq $name) {
-			return $self->{repository}->get_page($ns.$prev)
+			return $self->{store}->get_page($ns.$prev)
 				if length $prev;
 			last;
 		}
@@ -481,6 +483,9 @@ sub set_parse_tree { #warn "set_parse_tree from ", join(' ', caller), "\n" ;
 	my ($self, $tree) = @_;
 	$self->{status} = ''; # remove "new" or "deleted"
 	$self->{_links} = [ $self->list_links($tree) ];
+	Zim::Formats->fix_file_ending($tree);
+
+	# store tree
 	if (defined $self->{source}) {
 		my $io = $self->open_source('w')
 			|| die "Could not save parse tree, did not get an IO object.\n";
@@ -489,8 +494,8 @@ sub set_parse_tree { #warn "set_parse_tree from ", join(' ', caller), "\n" ;
 		#flock($io, LOCK_UN); # unlock
 		$io->close;
 		
-		$self->{repository}->_cache_page($self)
-			if $self->{repository}->can('_cache_page');
+		$self->{store}->_cache_page($self)
+			if $self->{store}->can('_cache_page');
 		# FIXME this hook does not belong here
 	}
 	else {
@@ -539,9 +544,8 @@ Returns a list with names of pages that link to this page.
 
 sub list_backlinks { # 'cloning' hack needed when exporting
 	my $obj = $_[0]->{cloning} ? $_[0]->{cloning} : $_[0];
-	my $rep = $obj->{repository}->root;
-	return $rep->can('list_backlinks')
-		? $rep->list_backlinks($obj) : () ;
+	return $obj->{root}->can('list_backlinks')
+		? $obj->{root}->list_backlinks($obj) : () ;
 }
 
 =item C<update_links(FROM => TO, ...)>
@@ -578,6 +582,10 @@ sub update_links {
 			# ergo, if "new_match" is there, this was not a link
 			# to "match" in the first place
 		my $new = $self->relative_name($links{$match});
+		unless (length $new) {
+			warn "\tBUG: could not update link to $links{$match}\n";
+			next;
+		}
 		warn "\tUpdating $match => $new\n";
 		$$ref[1]{to} = $new;
 		$$ref[2] = $new if @$ref == 3 and $$ref[2] eq $old;
@@ -599,9 +607,13 @@ sub update_links_self {
 		my ($t, $l) = $self->parse_link($old);
 		#warn "found link, old = $old => $t, $l\n";
 		next unless $t eq 'page';
-		my $page = $self->{repository}->root->resolve_name($l, $from);
+		my $page = $self->{root}->resolve_name($l, $from);
 		#warn "resolved $l => $page\n";
 		my $new = $self->relative_name($page);
+		unless (length $new) {
+			warn "\tBUG: could not update link to $page\n";
+			next;
+		}
 		warn "\tUpdating $old => $new\n";
 		$$ref[1]{to} = $new;
 		$$ref[2] = $new if @$ref == 3 and $$ref[2] eq $old;
@@ -653,7 +665,7 @@ sub relative_path {
 	my ($self, $file) = @_;
 	return $file if $file =~ m#^\w[\w\+\-\.]+:/#; # url
 
-	my $root = $self->{repository}->root->{dir};
+	my $root = $self->{root}{dir};
 	my $rel = Zim::File->rel_path($file, $root, 0);
 	return $file unless defined $rel; # file not below root
 
@@ -662,6 +674,33 @@ sub relative_path {
 	return defined($rel) ? $rel : $file ;
 }
 
+=item C<get_text()>
+
+Returns plain text page. Used by "Email Page" action.
+Not to be used for exporting.
+
+=cut
+
+sub get_text {
+	my $self = shift;
+	my $tree = $self->get_parse_tree;
+	my $fmt = 'Zim::Formats::Wiki';
+	my ($io, $text);
+	open $io, '>', \$text;
+	$fmt->save_tree($io, $tree, $self);
+	close $io;
+	return $text;
+}
+
+=item C<commit_change()>
+
+=item C<discard_change()>
+
+=cut
+
+sub commit_change { $_[0]->{source}->commit_change if $_[0]->{source} }
+
+sub discard_change { $_[0]->{source}->discard_change if $_[0]->{source} }
 
 1;
 
@@ -721,6 +760,6 @@ modify it under the same terms as Perl itself.
 =head1 SEE ALSO
 
 L<Zim>,
-L<Zim::Repository>
+L<Zim::Store>
 
 =cut
